import json


def load_json_or_dict(json_or_dict):
    if isinstance(json_or_dict, dict):
        return json_or_dict
    elif isinstance(json_or_dict, str):
        try:
            # try path actions on the passed variable
            with open(json_or_dict, 'r') as infile:
                loaded_json = json.load(infile)
            return loaded_json
        except FileNotFoundError:
            return {}


class JsonMAJ:
    def __init__(self, json_path, update_values={}, indent=4,
                 bids_null=False):
        self.json_path = json_path
        self.json_data = load_json_or_dict(self.json_path)
        self.update_values = load_json_or_dict(update_values)
        self.indent = indent
        self.bids_null = bids_null  # if set to true replace null's with "none" per bids specification

    def update(self, values=None):
        try:
            with open(self.json_path, 'r') as infile:
                try:
                    self.json_data = json.load(infile)
                except ValueError:
                    pass
        except FileNotFoundError:
            pass

        if values:
            self.json_data.update(values)
        else:
            self.json_data.update(self.update_values)

        with open(self.json_path, 'w') as outfile:
            json.dump(self.json_data, outfile, indent=self.indent, default=str)

        if self.bids_null:
            with open(self.json_path, 'r') as infile:
                json_strings = infile.read()
                infile.close()

            if json_strings:
                json_strings = json_strings.replace('null', '"none"')
                json_strings = json.loads(json_strings)
                self.json_data.update(json_strings)

                with open(self.json_path, 'w') as outfile:
                    json.dump(self.json_data, outfile, indent=self.indent, default=str)

    def remove(self, *keys):
        for key in keys:
            self.json_data.pop(key, None)

        with open(self.json_path, 'w') as outfile:
            json.dump(self.json_data, outfile, indent=self.indent, default=str)

    def get(self, *items):
        data_copy = self.json_data.copy()
        try:
            for item in items:
                data_copy = data_copy[item]
        except KeyError:
            data_copy = None
        return data_copy



import argparse
from json_maj.main import JsonMAJ
import sys
import ast


def very_tolerant_literal_eval(value):
    """
    Evaluates a string or string like input into a python datatype. Provides a lazy way to extract True from 'true',
    None from 'none', [0] from '[0'], etc. etc.
    :param value: the value you wish to convert to a python type
    :type value: string like, could be anything that can be evaluated as valid python
    :return: the value converted into a python object
    :rtype: depends on what ast.literal_eval determines the object to be
    """
    try:
        value = ast.literal_eval(value)
    except (SyntaxError, ValueError):
        if str(value).lower() == 'none':
            value = None
        elif str(value).lower() == 'true':
            value = True
        elif str(value).lower() == 'false':
            value = False
        elif str(value)[0] == '[' and str(value)[-1] == ']':
            array_contents = str(value).replace('[', '').replace(']', '')
            array_contents = array_contents.split(',')
            array_contents = [str.strip(content) for content in array_contents]
            # evaluate array contents one by one
            value = [very_tolerant_literal_eval(v) for v in array_contents]
        else:
            value = str(value)
    return value

class ParseKwargs(argparse.Action):
    """
    Class that is used to extract key pair arguments passed to an argparse.ArgumentParser objet via the command line.
    Accepts key value pairs in the form of 'key=value' and then passes these arguments onto the arg parser as kwargs.
    This class is used during the construction of the ArgumentParser class via the add_argument method. e.g.:\n
    `ArgumentParser.add_argument('--kwargs', '-k', nargs='*', action=helper_functions.ParseKwargs, default={})`
    """

    def __call__(self, parser, namespace, values, option_string=None):
        setattr(namespace, self.dest, dict())
        for value in values:
            try:
                key, value = value.split('=')
                getattr(namespace, self.dest)[key] = very_tolerant_literal_eval(value)
            except ValueError:
                raise Exception(f"Unable to unpack {value}")



parser = argparse.ArgumentParser()
parser.add_argument("json", metavar="json_file", help="Json file to be updated")
parser.add_argument("--updatefile", "-f", required=False,
                    help="Json file to update from, values in this file will be inserted into "
                         "the file passed as the first argument for this cli.")
parser.add_argument('--kwargs', '-k', nargs='*', action=ParseKwargs, default={},
                    help="Include additional values into a json or override values extracted from "
                         "the supplied json. e.g. including `--kwargs TimeZero='12:12:12'` would override the "
                         "calculated TimeZero. Any number of additional arguments can be supplied after --kwargs "
                         "e.g. `--kwargs BidsVariable1=1 BidsVariable2=2` etc etc.")
parser.add_argument('--get', '-g', nargs='+', help="Checks for the value of of a key within the json.")
parser.add_argument('--remove', '-r', nargs='+', required=False,
                    help="Provide a key or several keys to be removed from a json in the form of `--remove key1 key2`. "
                         "This argument can not be used with arguments that add information to a json.")


def cli():
    args = parser.parse_args()
    if (args.remove and args.updatefile) or (args.remove and args.kwargs):
        print("You must either remove data from the json or add it, you cannot do both at the same time")
        sys.exit(1)

    if args.updatefile:
        json_maj = JsonMAJ(json_path=args.json,
                           update_values=args.updatefile)
        json_maj.update()

    if args.kwargs:
        json_maj = JsonMAJ(json_path=args.json,
                           update_values=args.kwargs)
        json_maj.update()

    if args.remove:
        json_maj = JsonMAJ(json_path=args.json)
        for remove in args.remove:
            json_maj.remove(remove)

    if args.get:
        json_maj = JsonMAJ(json_path=args.json)
        print(json_maj.get(*args.get))


if __name__ == "__main__":
    cli()
    print('debug')



#! /usr/bin/env python3
import pathlib
import subprocess
import os
import platform
import toml
import shutil


# collect the path to this script
this_scripts_path = pathlib.Path(__file__)

# the build path for any executable created via pyinstaller should be in ../build/
build_folder = os.path.join(this_scripts_path.parent.parent, 'build')
project_folder = this_scripts_path.parent.parent

# check if pyinstaller is installed
check_for_pyinstaller = subprocess.run("pyinstaller --help", shell=True, stdout=subprocess.DEVNULL)

# get architecture and platform
platform_and_arch = platform.platform()

# collect version of software
project_toml = toml.load(f"{this_scripts_path.parent.parent}/pyproject.toml")
version = project_toml['tool']['poetry']['version']
executable_name = f"jsonmaj-{version}_{platform_and_arch}"

if check_for_pyinstaller.returncode == 0:
    command = f"pyinstaller cli.py -F -n {executable_name} --specpath {build_folder}"
    create_executable = subprocess.run(command, shell=True)
else:
    raise Exception("Pyinstaller is not installed.")

print("done")

if create_executable.returncode == 0:
    for folder in ['build', 'dist']:
        shutil.copytree(folder, build_folder, symlinks=False, ignore=None, ignore_dangling_symlinks=False,
                        dirs_exist_ok=True)

        # remove files b/c why not
        shutil.rmtree(folder)

    # lastly place the binary in the binary folder
    try:
        # shutil is garbage
        shutil.move(os.path.join(build_folder, executable_name, executable_name), os.path.join(project_folder, 'executable'))
    except shutil.Error:
        os.remove(os.path.join(project_folder, 'executable', executable_name))
        shutil.move(os.path.join(build_folder, executable_name, executable_name),
                    os.path.join(project_folder, 'executable'))